#!/usr/bin/perl -w
#
# notify_by_snmp.pl - Notification script for ITRS OP5 Monitor
#                     that sends notifications as SNMP traps.
#
# Author: Mathias Sundman <ms@op5.se>
#
# Version: 2.0 2007-02-21
#
# Copyright(C) 2007 OP5 AB
# All rights reserved.
#
# History:
#
# v2.0 2007-02-21
#   Complete rewrite to send all objects specified in the Nagios MIB.
#
# v1.0 2007-02-20
#   Initial release
#
#
# This script requires SNMP_util.pm, available from:
# http://www.switch.ch/misc/leinen/snmp/perl/

use strict;
use POSIX;
use SNMP_util "0.54";

sub print_usage ();
sub is_hostname ;
sub parse_args ();
sub logit ;
  
$ENV{'PATH'}='';
$ENV{'BASH_ENV'}='';
$ENV{'ENV'}='';

my $PROGNAME = "notify_by_snmp.pl";
my $logfile = "/tmp/notify_by_snmp.log";
  
# defaults
my $community = "public";
my $snmp_version = 1;
my $port = 162;
my $enterpriseOID = ".1.3.6.1.4.1.20006.1"; 	# NAGIOS-NOTIFY-MIB::nagiosNotify
my $generalID="6"; 				# enterprice Specific
my $specificID="1";
my $debug=0;

# variables
my %val;
my $hostname;
my $type;


parse_args();

# Import MIBS
if (snmpMIB_to_OID("/usr/share/snmp/mibs/NAGIOS-ROOT-MIB.txt") < 1) {
    logit "/usr/share/snmp/mibs/NAGIOS-ROOT-MIB.txt not found\n";
	exit(1);
}
if (snmpMIB_to_OID("/usr/share/snmp/mibs/NAGIOS-NOTIFY-MIB.txt") < 1) {
    logit "/usr/share/snmp/mibs/NAGIOS-NOTIFY-MIB.txt not found\n";
	exit(1);
}

if($val{"NOTIFICATIONTYPE"} eq "PROBLEM"){ $val{"NOTIFICATIONTYPE"}=0}
if($val{"NOTIFICATIONTYPE"} eq "RECOVERY"){ $val{"NOTIFICATIONTYPE"}=1}
if($val{"NOTIFICATIONTYPE"} eq "ACKNOWLEDGEMENT"){ $val{"NOTIFICATIONTYPE"}=2}
if($val{"NOTIFICATIONTYPE"} eq "FLAPPINGSTART"){ $val{"NOTIFICATIONTYPE"}=3}
if($val{"NOTIFICATIONTYPE"} eq "FLAPPINGSTOP"){ $val{"NOTIFICATIONTYPE"}=4}

if($val{"NOTIFICATIONTYPE"} eq "OK"){ $val{"NOTIFICATIONTYPE"}=0}
if($val{"NOTIFICATIONTYPE"} eq "WARNING"){ $val{"NOTIFICATIONTYPE"}=1}
if($val{"NOTIFICATIONTYPE"} eq "CRITICAL"){ $val{"NOTIFICATIONTYPE"}=2}
if($val{"NOTIFICATIONTYPE"} eq "UNKNOWN"){ $val{"NOTIFICATIONTYPE"}=3}

if($val{"HOSTSTATETYPE"} eq "UP"){ $val{"HOSTSTATETYPE"}=0}
if($val{"HOSTSTATETYPE"} eq "DOWN"){ $val{"HOSTSTATETYPE"}=1}
if($val{"HOSTSTATETYPE"} eq "UNREACHABLE"){ $val{"HOSTSTATETYPE"}=2}

# Send SNMP trap!
if ($type eq "nHostEvent") {		##### nHostEvent #####
	snmptrap("$community\@$hostname:$port", $enterpriseOID . ".5", "$hostname", $generalID, $specificID, 
		 "nHostname", 		"string", 	$val{"HOSTNAME"},
		 "nHostStateID", 	"integer",	$val{"HOSTSTATEID"},
		 "nHostStateType", 	"integer",	$val{"HOSTSTATETYPE"},
		 "nHostAttempt", 	"integer32",	$val{"HOSTATTEMPT"},
		 "nHostDurationSec", 	"integer32",	$val{"HOSTDURATIONSEC"},
		 "nHostGroupName",	"string",	$val{"HOSTGROUPNAME"},
		 "nHostLastCheck",	"integer",	$val{"LASTHOSTCHECK"},
		 "nHostLastChange",	"integer",	$val{"LASTHOSTSTATECHANGE"},
		 "nHostOutput",		"string", 	$val{"HOSTOUTPUT"});
}
elsif ($type eq "nHostNotify") {	##### nHostNotify #####
	snmptrap("$community\@$hostname:$port", $enterpriseOID . ".6", "$hostname", $generalID, $specificID, 
		 "nHostNotifyType",	"integer", 	$val{"NOTIFICATIONTYPE"},
		 "nHostNotifyNum",	"integer32", 	$val{"NOTIFICATIONNUMBER"},
		 "nHostAckAuthor",	"string", 	$val{"HOSTACKAUTHOR"},
		 "nHostAckComment",	"string", 	$val{"HOSTACKCOMMENT"},
		 "nHostname", 		"string", 	$val{"HOSTNAME"},
		 "nHostStateID", 	"integer",	$val{"HOSTSTATEID"},
		 "nHostStateType", 	"integer",	$val{"HOSTSTATETYPE"},
		 "nHostAttempt", 	"integer32",	$val{"HOSTATTEMPT"},
		 "nHostDurationSec", 	"integer32",	$val{"HOSTDURATIONSEC"},
		 "nHostGroupName",	"string",	$val{"HOSTGROUPNAME"},
		 "nHostLastCheck",	"integer",	$val{"LASTHOSTCHECK"},
		 "nHostLastChange",	"integer",	$val{"LASTHOSTSTATECHANGE"},
		 "nHostOutput",		"string", 	$val{"HOSTOUTPUT"});
}
elsif ($type eq "nSvcEvent") {		##### nSvcEvent #####
	snmptrap("$community\@$hostname:$port", $enterpriseOID . ".7", "$hostname", $generalID, $specificID, 
		 "nHostname", 		"string", 	$val{"HOSTNAME"},
		 "nHostStateID", 	"integer",	$val{"HOSTSTATEID"},
		 "nSvcDesc",	 	"string",	$val{"SERVICEDESCRIPTION"},
		 "nSvcStateID", 	"integer",	$val{"SERVICESTATEID"},
		 "nSvcAttempt", 	"integer32",	$val{"SERVICEATTEMPT"},
		 "nSvcDurationSec", 	"integer32",	$val{"SERVICEDURATIONSEC"},
		 "nSvcGroupName",	"string",	$val{"SERVICEGROUPNAME"},
		 "nSvcLastCheck",	"integer",	$val{"LASTSERVICECHECK"},
		 "nSvcLastChange",	"integer",	$val{"LASTSERVICESTATECHANGE"},
		 "nSvcOutput",		"string", 	$val{"SERVICEOUTPUT"});
}
elsif ($type eq "nSvcNotify") {		##### nSvcNotify #####
	snmptrap("$community\@$hostname:$port", $enterpriseOID . ".8", "$hostname", $generalID, $specificID, 
		 "nSvcNotifyType",	"integer", 	$val{"NOTIFICATIONTYPE"},
		 "nSvcNotifyNum",	"integer32", 	$val{"NOTIFICATIONNUMBER"},
		 "nSvcAckAuthor",	"string", 	$val{"SERVICEACKAUTHOR"},
		 "nSvcAckComment",	"string", 	$val{"SERVICEACKCOMMENT"},
		 "nHostname", 		"string", 	$val{"HOSTNAME"},
		 "nHostStateID", 	"integer",	$val{"HOSTSTATEID"},
		 "nSvcDesc",	 	"string",	$val{"SERVICEDESCRIPTION"},
		 "nSvcStateID", 	"integer",	$val{"SERVICESTATEID"},
		 "nSvcAttempt", 	"integer32",	$val{"SERVICEATTEMPT"},
		 "nSvcDurationSec", 	"integer32",	$val{"SERVICEDURATIONSEC"},
		 "nSvcGroupName",	"string",	$val{"SERVICEGROUPNAME"},
		 "nSvcLastCheck",	"integer",	$val{"LASTSERVICECHECK"},
		 "nSvcLastChange",	"integer",	$val{"LASTSERVICESTATECHANGE"},
		 "nSvcOutput",		"string", 	$val{"SERVICEOUTPUT"});
}
else {
	logit "Unknown notification type!\n"; # This should not happend!
	print "Unknown notification type!\n"; # This should not happend!
	exit(1);
}


### subs ###
sub print_usage () {
        print "Usage: $PROGNAME -h <host> -t <notification-typ> [-C community] variables...

  -h, --hostname=HOST            Name or IP address of host to send SNMP trap to
  -C, --community=COMMUNITY      snmp community string (default: public)
  -t, --type=notifitcation-type  Notification Type 
	
  notification-type should be (nHostEvent | nHostNotify | nSvcEvent | nSvcNotify)
  Freeform variables should be specified as 'VARIABLE=VALUE'.
";
	exit(1);

}

sub parse_args()
{
	my ($k, $v, $i);

	# parse arguments
	if($#ARGV < 2) {
	        print_usage();
	}
	for($i=0; $i<=$#ARGV; $i++) {
	        if($ARGV[$i] =~ /^-h|^--hostname/) {
	                $hostname = $ARGV[++$i];
	                #$val{"HOSTNAME"} = $ARGV[$i];
	        }
	        elsif($ARGV[$i] =~ /^-C|^--community/) {
	                $community = $ARGV[++$i];
	                #$val{"COMMUNITY"} = $ARGV[$i];
	        }
	        elsif($ARGV[$i] =~ /^-t|^--type/) {
	                $type = $ARGV[++$i];
	        }
	        elsif($ARGV[$i] =~ /^-d|^--debug/) {
	                $debug = 1;
	        }
	        elsif($ARGV[$i] =~ /^([^=]*)=(.*)$/) {
	                # this is a free form variable, so register it
	                $k = $1;
	                $v = $2;
	                $val{$k} = $v;
	        }
	        else {
	                print "Unknown option: " . $ARGV[$i] . "\n\n";
	                logit "Unknown option: " . $ARGV[$i] . "\n\n";
	                print_usage();
	        }
	}
	
	unless (defined $hostname) {
	        print "\nNo target hostname specified.\n";
	        logit "No target hostname specified.\n";
	        exit(1);
	}
	if (! is_hostname($hostname)){
	        print "$hostname is not a valid hostname.\n";
	        logit "$hostname is not a valid hostname.\n";
	        exit(1);
	}
	
	unless (defined $type) {
	        logit "No notification-type specified.\n";
	        print "\nNo notification-type specified.\n";
	        exit(1);
	}

	if ($type eq "nHostEvent") {
		unless (defined $val{"HOSTNAME"}) {
	        	logit "No HOSTNAME variable specified.\n";
	        	print "\nNo HOSTNAME variable specified.\n";
		        exit(1);
		}
		unless (defined $val{"HOSTSTATEID"}) {
			$val{"HOSTSTATEID"} = 0;
		}
		unless (defined $val{"HOSTSTATETYPE"}) {
			$val{"HOSTSTATETYPE"} = 0;
		}
		unless (defined $val{"HOSTATTEMPT"}) {
			$val{"HOSTATTEMPT"} = 0;
		}
		unless (defined $val{"HOSTDURATIONSEC"}) {
			$val{"HOSTDURATIONSEC"} = 0;
		}
		unless (defined $val{"HOSTGROUPNAME"}) {
			$val{"HOSTGROUPNAME"} = "";
		}
		unless (defined $val{"LASTHOSTCHECK"}) {
			$val{"LASTHOSTCHECK"} = 0;
		}
		unless (defined $val{"LASTHOSTSTATECHANGE"}) {
			$val{"LASTHOSTSTATECHANGE"} = 0;
		}
		unless (defined $val{"HOSTOUTPUT"}) {
			$val{"HOSTOUTPUT"} = "<No output from host>";
		}
	}
	elsif ($type eq "nHostNotify") {
		unless (defined $val{"NOTIFICATIONTYPE"}) {
	        	logit "No NOTIFICATIONTYPE variable specified.\n";
	        	print "\nNo NOTIFICATIONTYPE variable specified.\n";
		        exit(1);
		}
		unless (defined $val{"NOTIFICATIONNUMBER"}) {
			$val{"NOTIFICATIONNUMBER"} = 0;
		}
		unless (defined $val{"HOSTACKAUTHOR"}) {
			$val{"HOSTACKAUTHOR"} = "";
		}
		unless (defined $val{"HOSTACKCOMMENT"}) {
			$val{"HOSTACKCOMMENT"} = "";
		}
		unless (defined $val{"HOSTNAME"}) {
	        	logit "No HOSTNAME variable specified.\n";
	        	print "\nNo HOSTNAME variable specified.\n";
		        exit(1);
		}
		unless (defined $val{"HOSTSTATEID"}) {
			$val{"HOSTSTATEID"} = 0;
		}
		unless (defined $val{"HOSTSTATETYPE"}) {
			$val{"HOSTSTATETYPE"} = 0;
		}
		unless (defined $val{"HOSTATTEMPT"}) {
			$val{"HOSTATTEMPT"} = 0;
		}
		unless (defined $val{"HOSTDURATIONSEC"}) {
			$val{"HOSTDURATIONSEC"} = 0;
		}
		unless (defined $val{"HOSTGROUPNAME"}) {
			$val{"HOSTGROUPNAME"} = "";
		}
		unless (defined $val{"LASTHOSTCHECK"}) {
			$val{"LASTHOSTCHECK"} = 0;
		}
		unless (defined $val{"LASTHOSTSTATECHANGE"}) {
			$val{"LASTHOSTSTATECHANGE"} = 0;
		}
		unless (defined $val{"HOSTOUTPUT"}) {
			$val{"HOSTOUTPUT"} = "<No output from host>";
		}
	}
	elsif ($type eq "nSvcEvent") {
		unless (defined $val{"HOSTNAME"}) {
	        	logit "No HOSTNAME variable specified.\n";
	        	print "\nNo HOSTNAME variable specified.\n";
		        exit(1);
		}
		unless (defined $val{"HOSTSTATEID"}) {
			$val{"HOSTSTATEID"} = 0;
		}
		unless (defined $val{"SERVICEDESCRIPTION"}) {
	        	logit "No SERVICEDESCRIPTION variable specified.\n";
	        	print "\nNo SERVICEDESCRIPTION variable specified.\n";
		        exit(1);
		}
		unless (defined $val{"SERVICESTATEID"}) {
			$val{"SERVICESTATEID"} = 0;
		}
		unless (defined $val{"SERVICEATTEMPT"}) {
			$val{"SERVICEATTEMPT"} = 0;
		}
		unless (defined $val{"SERVICEDURATIONSEC"}) {
			$val{"SERVICEDURATIONSEC"} = 0;
		}
		unless (defined $val{"SERVICEGROUPNAME"}) {
			$val{"SERVICEGROUPNAME"} = "";
		}
		unless (defined $val{"LASTSERVICECHECK"}) {
			$val{"LASTSERVICECHECK"} = 0;
		}
		unless (defined $val{"LASTSERVICESTATECHANGE"}) {
			$val{"LASTSERVICESTATECHANGE"} = 0;
		}
		unless (defined $val{"SERVICEOUTPUT"}) {
			$val{"SERVICEOUTPUT"} = "<No output from service>";
		}
	}
	elsif ($type eq "nSvcNotify") {
		unless (defined $val{"NOTIFICATIONTYPE"}) {
	        	logit "No NOTIFICATIONTYPE variable specified.\n";
	        	print "\nNo NOTIFICATIONTYPE variable specified.\n";
		        exit(1);
		}
		unless (defined $val{"NOTIFICATIONNUMBER"}) {
			$val{"NOTIFICATIONNUMBER"} = 0;
		}
		unless (defined $val{"SERVICEACKAUTHOR"}) {
			$val{"SERVICEACKAUTHOR"} = "";
		}
		unless (defined $val{"SERVICEACKCOMMENT"}) {
			$val{"SERVICEACKCOMMENT"} = "";
		}
		unless (defined $val{"HOSTNAME"}) {
	        	logit "No HOSTNAME variable specified.\n";
	        	print "\nNo HOSTNAME variable specified.\n";
		        exit(1);
		}
		unless (defined $val{"HOSTSTATEID"}) {
			$val{"HOSTSTATEID"} = 0;
		}
		unless (defined $val{"SERVICEDESCRIPTION"}) {
	        	logit "No SERVICEDESCRIPTION variable specified.\n";
	        	print "\nNo SERVICEDESCRIPTION variable specified.\n";
		        exit(1);
		}
		unless (defined $val{"SERVICESTATEID"}) {
			$val{"SERVICESTATEID"} = 0;
		}
		unless (defined $val{"SERVICEATTEMPT"}) {
			$val{"SERVICEATTEMPT"} = 0;
		}
		unless (defined $val{"SERVICEDURATIONSEC"}) {
			$val{"SERVICEDURATIONSEC"} = 0;
		}
		unless (defined $val{"SERVICEGROUPNAME"}) {
			$val{"SERVICEGROUPNAME"} = "";
		}
		unless (defined $val{"LASTSERVICECHECK"}) {
			$val{"LASTSERVICECHECK"} = 0;
		}
		unless (defined $val{"LASTSERVICESTATECHANGE"}) {
			$val{"LASTSERVICESTATECHANGE"} = 0;
		}
		unless (defined $val{"SERVICEOUTPUT"}) {
			$val{"SERVICEOUTPUT"} = "<No output from service>";
		}
	}
	else {
		logit "Unknown notification-type.\n\n";
		print "Unknown notification-type.\n\n";
		print_usage();
		exit(1);
	}
}

sub is_hostname 
{
        my $host1 = shift;
        if ($host1 && $host1 =~ m/^([0-9]+\.[0-9]+\.[0-9]+\.[0-9]+|[a-zA-Z][-a-zA-Z0-9]+(\.[a-zA-Z][-a-zA-Z0-9]+)*)$/) {
                return 1;
        }else{
                return 0;
        }
}


sub logit {
    if($debug != 0) {
	my $msg = shift;
	my $now = localtime time;
	my $file = $logfile;
	open FILE, ">>$file" or die "unable to open $file $!";
	print FILE "$now - $msg";
    }
}
